@@ -6,6 +6,8 @@
 
 #include <leatherman/util/environment.hpp>
 #include <leatherman/util/option_set.hpp>
+#include <boost/optional.hpp>
+#include <boost/filesystem.hpp>
 
 #include <string>
 #include <vector>
@@ -355,7 +357,8 @@ namespace leatherman { namespace execution {
         lth_util::option_set<execution_options> const& options = { execution_options::trim_output, execution_options::merge_environment, execution_options::redirect_stderr_to_null });
 
     /**
-     * Executes the given program by writing the output of stdout and stderr to specified files.
+     * Executes the given program by writing the output of stdout and stderr to specified files. The output
+     * is processed line-by-line, so binary data isn't supported.
      * @param file The name or path of the program to execute.
      * @param arguments The arguments to pass to the program. On Windows they will be quoted as needed for spaces.
      * @param input A string to place on stdin for the child process before reading output.
@@ -364,7 +367,7 @@ namespace leatherman { namespace execution {
      * @param environment The environment variables to pass to the child process.
      * @param pid_callback The callback that is called with the pid of the child process. Defaults to no callback, in which case the pid will not be processed.
      * @param timeout The timeout, in seconds. Defaults to no timeout.
-     * @param options The execution options.  Defaults to trimming output and merging the environment.
+     * @param options The execution options. Defaults to trimming output and merging the environment.
      * @return Returns a result struct that will not contain the output of the streams for which a file was specified.
      *
      * Throws an execution_exception error in case it fails to open a file.
@@ -380,6 +383,36 @@ namespace leatherman { namespace execution {
         uint32_t timeout = 0,
         lth_util::option_set<execution_options> const& options = { execution_options::trim_output, execution_options::merge_environment });
 
+    /**
+     * Executes the given program by writing the output of stdout and stderr to specified files. The output
+     * is processed line-by-line, so binary data isn't supported.
+     * @param file The name or path of the program to execute.
+     * @param arguments The arguments to pass to the program. On Windows they will be quoted as needed for spaces.
+     * @param input A string to place on stdin for the child process before reading output.
+     * @param out_file The file where the output on stdout will be written.
+     * @param err_file The file where the output on stderr will be written.
+     * @param environment The environment variables to pass to the child process.
+     * @param pid_callback The callback that is called with the pid of the child process.
+     * @param timeout The timeout, in seconds.
+     * @param perms The file permissions to apply when creating the out_file and err_file.
+     *              On Windows this only toggles read-only.
+     * @param options The execution options. Defaults to trimming output and merging the environment.
+     * @return Returns a result struct that will not contain the output of the streams for which a file was specified.
+     *
+     * Throws an execution_exception error in case it fails to open a file.
+     */
+    result execute(
+        std::string const& file,
+        std::vector<std::string> const& arguments,
+        std::string const& input,
+        std::string const& out_file,
+        std::string const& err_file,
+        std::map<std::string, std::string> const& environment,
+        std::function<void(size_t)> pid_callback,
+        uint32_t timeout,
+        boost::optional<boost::filesystem::perms> perms,
+        lth_util::option_set<execution_options> const& options = { execution_options::trim_output, execution_options::merge_environment });
+
     /**
      * Executes the given program and returns each line of output.
      * @param file The name or path of the program to execute.
